/*

    Island Mandel, a puzzle game taking place on the Mandelbrot-set
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef GIMANDEL_CONTROLLER_HPP
#define GIMANDEL_CONTROLLER_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Geometry.hpp"

#include <gtk/gtk.h>

#include <cassert>

namespace islandmandel
{
class Fractal;
class Figure;
namespace gimandel
{

class MapDisplay;
class GameWindow;

/* ************************************************************************** */
/* **************************** Controller ********************************** */
/* ************************************************************************** */

/**
 * The component processing user interaction, that is, catching all the signals
 * and transforming the fractal/figure accordingly.
 */
class Controller
{

  /* ************************************************************************ */
  public:

    /**
     * Create it for a given MapDisplay; fractal/figure is set empty.
     * installs the signal handlers.
     * @param g The game window as main controller
     * @param m The map
     */
    Controller(GameWindow& g, MapDisplay& m);

    /**
     * Exchange/set the figure, places it in the center of the screen
     * @param f The new figure
     */
    void setFigure(Figure& f);

    /**
     * Exchange/set the fractal
     * @param f The new fractal
     */
    inline void setFractal(Fractal& f)
    {
      fract=&f;
      assert(fract);
    }

  /* ************************************************************************ */
  private:

    // References needed
    GameWindow& game;
    MapDisplay& map;
    Fractal* fract;
    Figure* fig;

    /** Reference position for rotation/scaling */
    Coordinate rotscalStart;

    /**
     * Process a rotation/scaling command.
     * @param pos Second click is here
     */
    void doRotScal(const Coordinate& pos);

    // Signal handlers
    static gboolean handle_keyPress(GtkWidget* w, GdkEventKey* evt,
      gpointer ptr);
    static gboolean handle_mouseRelease(GtkWidget* w, GdkEventButton* evt,
      gpointer ptr);
    static gboolean handle_mousePress(GtkWidget* w, GdkEventButton* evt,
      gpointer ptr);
    static gboolean handle_mouseDrag(GtkWidget* w, GdkEventMotion* evt,
      gpointer ptr);

};

} // namespace gimandel
} // namespace islandmandel

#endif // Header-guard
