/*

    Island Mandel, a puzzle game taking place on the Mandelbrot-set
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Checker.hpp"

#include <cmath>

namespace islandmandel
{

/* ************************************************************************** */
/* ****************************** Checker *********************************** */
/* ************************************************************************** */

Figure::colourT Checker::getNumberColours() const
{
  return numCol;
}

void Checker::calcArea(floatCoordT tx, floatCoordT ty,
  floatCoordT sx, floatCoordT sy, Coordinate::compT w, Coordinate::compT h,
  Figure::colourT* px, Coordinate::compT rs) const
{
  /* We round the point-coordinates towards minus Infinity, then add them,
   * and do the result mod 2.  This should give 0 or 1 depending on the "square"
   * the point belongs to.
   */

  for(Coordinate::compT y(0); y!=h; ++y)
  {
    Figure::colourT* const line(px+y*rs);
    const Coordinate::compT roundedY(std::floor(ty+y*sy));
    for(Coordinate::compT x(0); x!=w; ++x)
    {
      const Coordinate::compT roundedX(std::floor(tx+x*sx));
      int curCol((roundedY+roundedX)%numCol);
      if(curCol<0)
        curCol+=numCol*(1+(-curCol)/numCol);
      assert(curCol>=0);
      line[x]=curCol;
    }
  }
}

} // namespace islandmandel
