/*

    Island Mandel, a puzzle game taking place on the Mandelbrot-set
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Geometry.hpp"

namespace islandmandel
{

/* ************************************************************************** */
/* ******************************* Transformation *************************** */
/* ************************************************************************** */

Coordinate Transformation::operator()(const Coordinate& c) const
{
  const entryT newX(c.getX()*matrix[0][0]+c.getY()*matrix[0][1]);
  const entryT newY(c.getX()*matrix[1][0]+c.getY()*matrix[1][1]);

  return Coordinate(newX, newY);
}

Transformation operator*(const Transformation& a, const Transformation& b)
{
  Transformation result; // Coordinates don't matter, get overwritten
  for(int i(0); i!=2; ++i)
    for(int j(0); j!=2; ++j)
      result.matrix[i][j]=
        a.matrix[i][0]*b.matrix[0][j]+
        a.matrix[i][1]*b.matrix[1][j];
  return result;
}

Transformation Transformation::forScaling(scaleFactT f)
{
  return Transformation(f, 0, 0, f);
}

Transformation Transformation::forRotation(angleT alpha)
{
  return Transformation(
    std::cos(alpha), -std::sin(alpha),
    std::sin(alpha), std::cos(alpha));
}

} // namespace islandmandel
