/*

    Island Mandel, a puzzle game taking place on the Mandelbrot-set
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Simple program to do a repeated calculation of Mandelbrot-sets, with a
 * hardcoded zooming/moving sequence.  Used for profiling/timing the Mandelbrot
 * calculation.
 */

#include "Fractal.hpp"
#include "CSet.hpp"

#include <cstdlib>
#include <cassert>
#include <iostream>
#include <string>

using namespace islandmandel;

// Size of the fractal to calculate
#define WIDTH 640
#define HEIGHT 480

int main()
{
  // Create the fractal
  Mandelbrot fract(WIDTH, HEIGHT);
  fract.calculate();

  // Perform the operations
  while(true)
  {
    std::string cmd;
    std::cin >> cmd;

    if(!std::cin)
      break;

    if(cmd=="move")
    {
      int dx, dy;
      std::cin >> dx >> dy;
      fract.move(dx, dy);
    } else if(cmd=="zoom")
    {
      float f;
      std::cin >> f;
      fract.zoom(f);
    } else if(cmd=="movezoom")
    {
      int dx, dy;
      float f;
      std::cin >> dx >> dy >> f;
      fract.moveZoom(dx, dy, f);
    } else
      std::cerr << "Unknown modification: " << cmd << std::endl;

    assert(std::cin);
  }

  return EXIT_SUCCESS;
}
